`timescale 1ns / 1ns

// note that DATA is only valid on the rising edge of lrck and it is
// left channel: bits 0:23 and right channel 24:47
module codec_receiver(RST, CLK,
                      BICK, LRCK,
                      SDIN, DATA);

   parameter WAIT_FOR_RISING_LRCK   = 0,
             READING_LEFT           = 1,
             WAIT_FOR_FALLING_LRCK  = 2,
             READING_RIGHT          = 3;
   
             
   input RST;
   input CLK;
   input BICK;
   input LRCK;
   input SDIN;
   
   wire  RST;
   wire  CLK;
   wire  BICK;
   wire  LRCK;
   wire  SDIN;

   output [0:47] DATA;
   reg [0:47]    DATA, next_data;
   
   reg   prev_bick, prev_lrck;
   wire  rising_bick, falling_lrck, rising_lrck;
   assign rising_bick = BICK && ~prev_bick;
   assign falling_lrck = ~LRCK && prev_lrck;
   assign rising_lrck = LRCK && ~prev_lrck;
   
   reg [0:5] counter, next_counter;
   reg [0:1] state, next_state;
   
   always @(posedge CLK)
     begin
        prev_bick <= BICK;
        prev_lrck <= LRCK;
        
        if(RST)
          begin
             counter <= 0;
             state <= 0;
             DATA <= 0;
          end
        else
          begin
             counter <= next_counter;
             state <= next_state;
             DATA <= next_data;
          end // else: !if(RST)
     end // always @ (posedge CLK)

   always @(*)
     begin
        next_counter = counter;
        next_state = state;
        next_data = DATA;
        case(state)
          WAIT_FOR_RISING_LRCK:
            begin
               if(rising_lrck)
                 begin
                    next_state = READING_LEFT;
                    next_counter = 0;
                 end
            end
          READING_LEFT:
            begin
               if(rising_bick)
                 begin
                    next_counter = counter + 1;
                    next_data = {DATA[1:47], SDIN};
                    if(counter == 23)
                      next_state = WAIT_FOR_FALLING_LRCK;
                 end
            end
          WAIT_FOR_FALLING_LRCK:
            begin
               if(falling_lrck)
                 begin
                    next_counter = 0;
                    next_state = READING_RIGHT;
                 end
            end
          READING_RIGHT:
            begin
               if(rising_bick)
                 begin
                    next_counter = counter + 1;
                    next_data = {DATA[1:47], SDIN};
                    if(counter == 23)
                      next_state = WAIT_FOR_RISING_LRCK;
                 end
            end
        endcase // case(state)
     end // always @ (*)
endmodule // codec_receiver

     