`timescale 1ns / 1ns

module opb_audio_controller(OPB_ABus,
                            OPB_BE,
                            OPB_Clk,
                            OPB_DBus,
                            OPB_RNW,
                            OPB_Rst,
                            OPB_select,
                            OPB_seqAddr,
                            
                            Sln_DBus,
                            Sln_errAck,
                            Sln_retry,
                            Sln_toutSup,
                            Sln_xferAck,

                            Interrupt,

                            XTAL_CLK,
                            SYS_CLK,
                            MCLK,
                            BICK,
                            LRCK,
                            SDTI1,
                            SDTI2,
                            SDTI3,
                            SDTI4,
                            SDTO,
                            PDN);

   // begin edk parameters
   parameter C_BASEADDR      = 32'hFFFFFF00;
   parameter C_HIGHADDR      = 32'hFFFFFFFF;
   parameter C_USER_ID_CODE  = 3;
   parameter C_OPB_AWIDTH    = 32;
   parameter C_OPB_DWIDTH    = 32;
   // end edk parameters

   // begin opb signals
   input [0:31] OPB_ABus;
   input [0:3]  OPB_BE;
   input        OPB_Clk;
   input [0:31] OPB_DBus;
   input        OPB_RNW;
   input        OPB_Rst;
   input        OPB_select;
   input        OPB_seqAddr;
   
   wire [0:31]  OPB_ABus;
   wire [0:3]   OPB_BE;
   wire         OPB_Clk;
   wire [0:31]  OPB_DBus;
   wire         OPB_RNW;
   wire         OPB_Rst;
   wire         OPB_select;
   wire         OPB_seqAddr;
   
   reg [0:31]   OPB_ABus_Reg;
   reg [0:3]    OPB_BE_Reg;
   reg [0:31]   OPB_DBus_Reg;
   reg          OPB_RNW_Reg;
   reg          OPB_select_Reg;
   reg          OPB_seqAddr_Reg;
   
   output [0:31] Sln_DBus;
   output        Sln_errAck;
   output        Sln_retry;
   output        Sln_toutSup;
   output        Sln_xferAck;
   
   wire [0:31]   Sln_DBus;
   wire          Sln_errAck;
   wire          Sln_retry;
   wire          Sln_toutSup;
   wire          Sln_xferAck;

   assign        Sln_errAck = 0;
   assign        Sln_retry = 0;
   assign        Sln_toutSup = 0;
   // end opb signals
   
   output        Interrupt;
   wire          Interrupt;

   // begin codec interface signals
   input         XTAL_CLK;
   output        SYS_CLK;
   output        MCLK;
   output        BICK;
   output        LRCK;
   output        SDTI1;
   output        SDTI2;
   output        SDTI3;
   output        SDTI4;
   input         SDTO;
   output        PDN;


   wire          XTAL_CLK;
   wire          SYS_CLK;
   wire          MCLK;
   wire          BICK;
   wire          LRCK;
   wire          SDTI1;
   wire          SDTI2;
   wire          SDTI3;
   wire          SDTI4;
   wire          SDTO;
   wire          PDN;
   // end codec interface signals

   reg           its_for_me;
   
   wire [0:31]   data_bus;
   assign        Sln_DBus = data_bus;
   
   wire          ack;
   assign        Sln_xferAck = ack;

   always @(posedge OPB_Clk)
     begin
        if(OPB_Rst)
          begin
             OPB_ABus_Reg <= 0;
             OPB_BE_Reg <= 0;
             OPB_DBus_Reg <= 0;
             OPB_RNW_Reg <= 0;
             OPB_select_Reg <= 0;
             OPB_seqAddr_Reg <= 0;
          end // if (OPB_Rst)
        else
          begin
             OPB_ABus_Reg <= OPB_ABus;
             OPB_BE_Reg <= OPB_BE;
             OPB_DBus_Reg <= OPB_DBus;
             OPB_RNW_Reg <= OPB_RNW;
             OPB_select_Reg <= OPB_select;
             OPB_seqAddr_Reg <= OPB_seqAddr;
          end // else: !if(OPB_Rst)
     end // always @ (posedge OPB_Clk)
   

   always @(*)
     begin
        if(!OPB_Rst && OPB_select_Reg && C_BASEADDR[31:8] == OPB_ABus_Reg[0:23])
          its_for_me = 1;
        else
          its_for_me = 0;
     end
          

   codec_interface ci
     (
      .RST(OPB_Rst),
      .CLK(OPB_Clk),
      .RNW(OPB_RNW_Reg),
      .EN(its_for_me),
      .DIN(OPB_DBus_Reg), 
	  .BE(OPB_BE_Reg),
      .AIN(OPB_ABus_Reg[24:31]),
      .DOUT(data_bus),
      .ACK(ack),
      .INTR(Interrupt),
      .BICK(BICK),
      .LRCK(LRCK),
      .SDTI1(SDTI1),
      .SDTI2(SDTI2),
      .SDTI3(SDTI3),
      .SDTI4(SDTI4),
      .SDTO(SDTO),
      .PDN(PDN)
      );

   codec_clocks_generator ccg
     (
      .XTAL_CLK(XTAL_CLK),
      .RST(OPB_Rst),
      .CLK(SYS_CLK),
      .MCLK(MCLK),
      .BICK(BICK),
      .LRCK(LRCK)
      );
   
endmodule // opb_audio_controller

      